

'''
	import vegan._interfaces.off as vegan_off
	vegan_off.off (
		essence_path = "essence.py"
	)
'''

import vegan.monetary.off as monetary_off
import vegan._interfaces.sanique.off as sanic_off

from vegan._essence import prepare_essence
from vegan._essence import run_script_from_file

def off (
	essence_path
):	
	vegan_essence = prepare_essence (
		run_script_from_file (essence_path)
	)
	if ("onsite" in vegan_essence ["monetary"]):
		mongo_process = monetary_off.off (
			vegan_essence = vegan_essence
		)
	
	sanic_off.off (
		vegan_essence = vegan_essence
	)
	
	




'''
	from vegan.organisms.sanique._ops.on as turn_on_sanique
	turn_on_sanique ()
'''

'''
	python3 --shutdown --pidfilepath /var/run/mongodb/mongod.pid
'''

import multiprocessing
import subprocess
import time
import os
import atexit
import shutil
import sys

from ..utilities.has_sanic_check import has_sanic_check

from .status import check_sanique_status

from vegan._essence import retrieve_essence
	

def floating_process (procedure, CWD, env):
	print ("procedure:", procedure)
	process = subprocess.Popen (
		procedure, 
		cwd = CWD,
		env = env
	)
	
	pid = process.pid
	
	print ("sanic pid:", pid)

def turn_on_sanique ():
	essence = retrieve_essence ()

	has_sanic_check ()

	the_status = check_sanique_status ()
	if (the_status == "on"):
		print ("sanic is already on")		
		return;

	harbor_port = essence ["harbor"] ["port"]
	harbor_path = essence ["harbor"] ["directory"]

	env_vars = os.environ.copy ()
	env_vars ["USDA_food"] = essence ["USDA"] ["food"]
	env_vars ["NIH_supp"] = essence ["NIH"] ["supp"]
	env_vars ['PYTHONPATH'] = ":".join (sys.path)
	
	process = floating_process (
		procedure = [
			"sanic",
			f'harbor:create',
			f'--port={ harbor_port }',
			f'--host=0.0.0.0',
			
			'--fast',
			'--factory',
			
			#'--dev'
		],
		CWD = harbor_path,
		env = env_vars
	)

	return;






#from vegan.organisms.monetary.clique import clique as monetary_clique
#from vegan.organisms.customs.clique import clique as customs_clique
#from vegan.organisms.sanique.clique import clique as sanic_clique

import vegan._ops.on as vegan_on
#import vegan._ops.off as vegan_off
#import vegan._ops.status as vegan_status

#from vegan._essence import prepare_essence_from_py_file

from vegan._essence import build_essence, retrieve_essence

from vegan._essence import essence

import somatic

import os
import json
import time
import pathlib
from os.path import dirname, join, normpath
import sys

import rich
import click

this_directory = pathlib.Path (__file__).parent.resolve ()	

def clique ():
	'''
		Check for essence here and then set them 
		implicitly.
	'''
	print ('essence check')
	build_essence ()
	


	@click.group ()
	def group ():
		pass

	@click.command ("help")
	def help ():	
		the_mix = str (normpath (join (this_directory, "../..")))
		
		somatic.start ({
			"directory": the_mix,
			"relative path": the_mix,
			"port": 20000,
			"static port": False,
			"verbose": 1
		})

		import time
		while True:
			time.sleep (1000)
	
	
	#
	#	vegan on
	#
	@click.command ("essence")
	def print_essence ():	
		essence = retrieve_essence ()
		rich.print_json (data = essence)
	
	#
	#	vegan on
	#
	@click.command ("on")
	def on ():		
		vegan_on.on ()

	
	
	'''
	@click.command ("off")
	@click.option (
		'--essence-path', 
		default = 'essence.py',
		help = 'vegan off --essence-path essence.py', 
		required = False
	)
	def off (essence_path):
		vegan_off.off (essence_path)
		time.sleep (2)
		print ('off')
		
	@click.command ("status")
	@click.option (
		'--essence-path', 
		default = 'essence.py',
		help = 'vegan off --essence-path essence.py', 
		required = False
	)
	def status (essence_path):
		vegan_status.status (essence_path)
		time.sleep (2)

		
	

	
	group.add_command (off)
	group.add_command (status)
	'''
	
	
	
	group.add_command (help)
	
	group.add_command (on)
	group.add_command (print_essence)
		
	#group.add_command (monetary_clique ())
	#group.add_command (sanic_clique ())
	#group.add_command (customs_clique ())
	
	group ()




#





'''
	import vegan.organisms._regula.on as vegan_on
	vegan_on.on (
		essence_path = "essence.py"
	)
'''

from vegan.organisms.sanique._ops.on import turn_on_sanique
from vegan.organisms.monetary.node.on import turn_on_monetary_node
	
from vegan._essence import retrieve_essence

import rich

def on ():	
	essence = retrieve_essence ()

	rich.print_json (data = essence)

	if ("onsite" in essence ["monetary"]):
		turn_on_monetary_node ()
		
	turn_on_sanique ()	
		
	'''
	sanic_on.on (
		vegan_essence = vegan_essence
	)
	'''




'''
	mongod --dbpath ./../_mongo_data --port 39000
'''

'''
	import vegan.monetary.status as monetary_status
	the_monetary_status = monetary_status.status (
		essence = essence
	)
	
	import time
	while True:
		time.sleep (1)
'''

'''	
	mongo_process.terminate ()

	#
	#	without this it might appear as if the process is still running.
	#
	import time
	time.sleep (2)
'''

from fractions import Fraction
import multiprocessing
import subprocess
import time
import os
import atexit

from pymongo import MongoClient
from pymongo.errors import ConnectionFailure
from pymongo.errors import ServerSelectionTimeoutError
import rich

from vegan._essence import receive_monetary_URL
from vegan._essence import retrieve_essence

import ships.cycle as cycle

def find_monetary_status (
	loop_limit = 1
):
	essence = retrieve_essence ()
	monetary_URL = essence ["monetary"] ["URL"]
	
	print ("checking if can connect to URL:", monetary_URL)	
	
	counter = 0
	
	def show (* positionals, ** keywords):
		nonlocal counter
		counter += 1
	
		print (f'connection attempt { counter }', positionals, keywords)
	
		try:
			client = MongoClient (monetary_URL, serverSelectionTimeoutMS=2000)
			client.server_info ()
			
			print ("	A connection to the monetary node was established!")
			print ()
			
			return "on"
			
		except ConnectionFailure:
			pass;
			
		print ("	A connection to the monetary node could not be established!\n")
		print ()
		
		if (counter == loop_limit):
			return "off"
		
		raise Exception ("")
		
	
	proceeds = cycle.loops (
		show, 
		cycle.presents ([ 1 ]),
		
		#
		#	this is the loop limit
		#
		loops = loop_limit,
		delay = Fraction (1, 1),
		
		records = 0
	)
	
	print ("The monetary is:", proceeds)
	
	
	return proceeds;

	





'''
	import vegan._interfaces.status as vegan_status
	vegan_status.status (
		essence_path = "essence.py"
	)
'''

import vegan.monetary.status as monetary_status
import vegan._interfaces.sanique.status as sanic_status

from vegan._essence import prepare_essence
from vegan._essence import run_script_from_file

import rich

def status (
	essence_path
):	
	vegan_essence = prepare_essence (
		run_script_from_file (
			essence_path
		)
	)

	if ("onsite" in vegan_essence ["monetary"]):
		local_mongo_status = monetary_status.status (
			vegan_essence = vegan_essence
		)
		
	the_sanic_status = sanic_status.status (
		vegan_essence = vegan_essence
	)
	
	the_status = {
		"monetary": {
			"local": local_mongo_status
		},
		"sanique": {
			"local": the_sanic_status
		}
	}
	
	print ()
	rich.print_json (data = {
		"status": the_status
	})
	
	return the_status